<?php

/**
 * Cryptor/yform class
 *
 * @author Marshall
 */
class cryptor_yform {

    const PLUGIN_NAME = 'yform';
    const CHECKBOX_STATUS_ENABLED = 'on';

    protected static $autoencryptExtensionPoints = ['REX_YFORM_SAVED'];
    protected static $autoencryptFieldTypes = ['text', 'textarea', 'email'];

    /**
     * Execute a extension point
     * @param <obj> $ep
     */
    public static function executeExtensionpoint(rex_extension_point $ep) {
        if ($ep->getSubject() instanceof Exception) {
            return;
        }

        // Get the config for this table name
        $config = self::getConfig($ep->getParam('table'));
        if (!is_array($config)) {
            return;
        }

        // Loop over each config of this table and callback the cryptor extension point
        foreach ($config as $eventType => $fieldnames) {
            $className = 'cryptor_yform_ep_' . strtolower($ep->getName());
            if ($ep->getName() === $eventType && class_exists($className)) {
                call_user_func([$className, 'execute'], $ep, $fieldnames);
            }
        }
    }

    /**
     * Execute the autodelete on all defined tables
     * @param <string> $tableName
     */
    public static function executeAutodelete($tableName = null) {
        // Execute on given table
        if (!empty($tableName)) {
            cryptor_yform_autodelete::execute($tableName);
        }

        // Execute on all defined tables
        else {
            $config = self::getConfig();
            foreach ($config as $tableName => $tableConfig) {
                cryptor_yform_autodelete::execute($tableName);
            }
        }
    }

    /**
     * Executes ajax request
     * @param type $type
     * @param type $params
     * @return type
     */
    public static function executeAjaxRequest($type, $params) {
        if ($type === 'backend_decrypt') {
            return json_encode(cryptor::decrypt($params));
        } else if ($type === 'backend_encrypt') {
            return json_encode(cryptor::encrypt($params));
        } else if ($type === 'field_names') {
            $yFormTable = rex_yform_manager_table::get($params);
            $config = self::getConfig($params);
            $fields = self::getEncyptedFieldLabels($config, $yFormTable);
            return json_encode($fields);
        }
    }

    /**
     * Returns all supported extension points
     * @return <array> $autoencryptExtensionPoints
     */
    public static function getAutoencryptExtensionPoints() {
        return static::$autoencryptExtensionPoints;
    }

    /**
     * Returns all supported encryption field types
     * @return <array> $autoencryptFieldTypes
     */
    public static function getAutoencryptFieldTypes() {
        return static::$autoencryptFieldTypes;
    }

    /**
     * Returns the plugin config
     * @return <mixed> $configArray || false
     */
    protected static function getConfig($tableName = null) {
        return rex_plugin::get(cryptor::ADDON_NAME, self::PLUGIN_NAME)->getConfig($tableName);
    }

    /**
     * Returns encrypted fields and their attributes
     * @param <array> $config
     * @param <object> $yFormTable
     * @return <array> $fields
     */
    private static function getEncyptedFieldLabels($config, $yFormTable) {
        $fields = [];
        foreach ($config as $extensionPoint) {
            if (!is_array($extensionPoint)) {
                continue;
            }
            foreach ($extensionPoint as $fieldname => $status) {
                if ($status === self::CHECKBOX_STATUS_ENABLED) {
                    $fields[] = [
                        'name' => $fieldname,
                        'label' => $yFormTable->getValueField($fieldname)->getLabel(),
                    ];
                }
            }
        }
        return $fields;
    }

}
